local M = {}

local function sign(val)
    if val > 0 then return 1 end
    if val < 0 then return -1 end
    return 0
end

local function clamp(val, min, max)
    return math.min(math.max(val, min), max)
end

local function maprange(val, srcMin, srcMax, dstMin, dstMax, clampOutput)
    if clampOutput ~= false then clampOutput = true end
    if type(dstMin) ~= "number" then dstMin = 0.0 end
    if type(dstMax) ~= "number" then dstMax = 1.0 end

    local normalized = (val - srcMin) / (srcMax - srcMin) -- 0.0 to 1.0 when inside [srcMin, srcMax]
    
    local dstRange = dstMax - dstMin
    local output = dstRange * normalized + dstMin
    
    if clampOutput then
        local cMin = math.min(dstMin, dstMax)
        local cMax = math.max(dstMin, dstMax)
        
        output = clamp(output, cMin, cMax)
    end
    
    return output
end

local function blend(val1, val2, factor)
    return val1 * (1.0 - factor) + val2 * factor
end

local function motorKwToTorque(kw, rpm)
    -- Pw * 9.549 / RPM; 9.549 is motor torque constant
    return ( kw * 1000 * 9.549 ) / rpm
end

local function motorTorqueToKw(torque, rpm)
    return ( ( torque * rpm ) / 9.549 ) / 1000
end

M.sign = sign
M.clamp = clamp
M.maprange = maprange
M.blend = blend
M.motorKwToTorque = motorKwToTorque
M.motorTorqueToKw = motorTorqueToKw

return M